#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
from __future__ import unicode_literals
#from __future__ import absolute_import
from __future__ import division

import sys
sys.dont_write_bytecode = True

from optparse import OptionParser
import os
import time
from flocker import FileLocker
from compat import __init, __to_bytes, __to_fspath, __to_str
from sudo import __exec_cmd, __makedirs
from keyvalue import __to_keyvalue, __to_keyvalue_str

from datetime import datetime
import re
import glob

from filecmp import dircmp
import hashlib
import errno
import stat

DEV_DEFAULT = "/dev/sr0"
LOCK_FILE_DEFAULT = "/var/lock/bdrec_src.lock"

FILE_DEV_STATUS = "/var/run/bdrec.dev_status"
FILE_REC_STATUS = "/var/run/bdrec.rec_status"

MEDIAINFO_CMD = "/usr/bin/dvd+rw-mediainfo"
GROWISOFS_CMD = "/usr/bin/growisofs"
SG_TURS_CMD = "/usr/bin/sg_turs"
UDFINFO_CMD = "/usr/bin/udfinfo"
EJECT_CMD = "/usr/bin/eject"
MOUNT_CMD = "/bin/mount"
UMOUNT_CMD = "/bin/umount"

ROOTFS_MNT = "/var/tmp/bdrec/rootfs_mnt"

REC_STATUS_READY = "ready"
REC_STATUS_RECORDING = "recording"
REC_STATUS_SUCCESS = "success"
REC_STATUS_FAILED = "failed"

DEV_STATUS_READY = "ready"
DEV_STATUS_NOTREADY = "not ready"

DISC_STATUS_UNSUPPORTEDMEDIA = "unsupported media"
DISC_STATUS_BLANK = "blank"
DISC_STATUS_APPENDABLE = "appendable"
DISC_STATUS_COMPLETE = "complete"
DISC_STATUS_CLOSED = DISC_STATUS_COMPLETE

SUPPORTEDMEDIA = ("41h")
#SUPPORTEDMEDIA = ("41h", "43h")

LASTWRITTENSUMMARY = "last_written_summary.txt"

LOG_FMT_STR = "%s -> %s."
LOG_FMT_VERIFY_OK_STR = "%s -> %s (%s)."
LOG_FMT_VERIFY_NG_STR = "%s -> %s (誤:%s, 正:%s)."
HASH_DIRECTORY = "Directory"
HASH_NOT_FOUND = "Not found"
HASH_INTERNAL_ERROR = "Internal Error"
FILE_VERIFICATION_OK = "File Verification OK"
FILE_VERIFICATION_NG = "File Verification NG"

def _list_files(dir_path, relpath, log, dir_alt_name):
    result = 0
    if relpath:
        dir_abspath = os.path.join(dir_path, relpath)
    else:
        dir_abspath = dir_path

    dir_list = sorted(os.listdir(__to_fspath(dir_abspath)))
    for file in dir_list:
        if relpath:
            file_relpath = os.path.join(relpath, file)
        else:
            file_relpath = file

        file_abspath = os.path.join(dir_abspath, file)

        if os.path.isdir(__to_fspath(file_abspath)):
            if _list_files(dir_path, file_relpath, log, dir_alt_name) != 0:
                result = 1
            continue

        if log:
            log.log(file_relpath)

    return result

def __list_files(dev, log, dev_alt_name):
    __makedirs(ROOTFS_MNT, exist_ok=True)
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "ro,noatime,unhide", dev, ROOTFS_MNT])
    if returncode !=0:
        print("mount failed", file=sys.stderr)
        return 1
    result = _list_files(ROOTFS_MNT, None, log, dir_alt_name=dev_alt_name)
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT])
    if returncode !=0:
        print("umount failed", file=sys.stderr)
        return 1
    return result

def list(dev, dev_alt_name="dev"):
    if not dev:
        dev = DEV_DEFAULT
    dev_status = get_dev_status(dev)
    if not dev_status:
        print("get_dev_status failed", file=sys.stderr)
        return 1
    if dev_status != DEV_STATUS_READY:
        print("dev_status may not ready", file=sys.stderr)
        return 1
    volid = get_volid(dev)
    if not volid:
        print("not volid", file=sys.stderr)
        return 1
    start_time = time.time()
    from bdreclog import BdrecLog
    process_log = BdrecLog(file=sys.stdout)
    if not process_log:
        print("not process_log", file=sys.stderr)
        return 1
    result = 0
    process_log.log("volid: %s" % volid)
    process_log.log("datetime: %s" % datetime.fromtimestamp(time.time()))
    process_log.log("### List Start ###")
    if __list_files(dev, log=process_log, dev_alt_name=dev_alt_name) != 0:
        print("__list_files failed", file=sys.stderr)
        result = 1
    end_time = time.time()
    elapsed_time = end_time - start_time
    process_log.log("datetime: %s" % datetime.fromtimestamp(end_time))
    if result == 0:
        process_log.log("%s: %s" % (volid, BdrecLog.SUCCESS))
    else:
        process_log.log("%s: %s" % (volid, BdrecLog.FAILED))
    return result

def check_writable_volid(volid):
    try:
        bytes_volid = __to_bytes(volid)
    except:
        print("volid can not encode to UTF-8", file=sys.stderr)
        return 1
    if not isinstance(bytes_volid, bytes):
        print("volid type not bytes", file=sys.stderr)
        return 1
    if len(bytes_volid) > 32:
        print("volid over 32 bytes", file=sys.stderr)
        print(str(len(bytes_volid)), file=sys.stderr)
        return 1
    return 0

def _get_ng_file_list(dcmp, path=[], ng_file_list=[]):
    for name in dcmp.diff_files:
        ng_file_list.append("/".join(path + [name]))
    for name in dcmp.left_only:
        ng_file_list.append("/".join(path + [name]))
    for name in dcmp.right_only:
        ng_file_list.append("/".join(path + [name]))
    #for sub_dcmp in dcmp.subdirs.values():
    for key in dcmp.subdirs.keys():
        sub_dcmp = dcmp.subdirs.get(key)
        ng_file_list = _get_ng_file_list(sub_dcmp, path + [key], ng_file_list)
    return ng_file_list

def __get_ng_file_list(dev, dir_path):
    __makedirs(ROOTFS_MNT, exist_ok=True)
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "ro,noatime,unhide", dev, ROOTFS_MNT])
    if returncode !=0:
        print("mount failed", file=sys.stderr)
        return None
    dcmp = dircmp(__to_fspath(ROOTFS_MNT), __to_fspath(dir_path), ignore=[])
    ng_file_list = _get_ng_file_list(dcmp)
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT])
    if returncode !=0:
        print("umount failed", file=sys.stderr)
        return None
    return ng_file_list

def __get_file_stat(file_path):
    try:
        st = os.stat(__to_fspath(file_path))
        if stat.S_ISDIR(st.st_mode):
            return HASH_DIRECTORY
        if stat.S_ISREG(st.st_mode):
            return "%d bytes, %s" % (st.st_size, datetime.fromtimestamp(int(st.st_mtime)))
        return HASH_INTERNAL_ERROR
    except OSError as e:
        if e.errno == errno.EISDIR:
            return HASH_DIRECTORY
        elif e.errno == errno.ENOENT:
            return HASH_NOT_FOUND
        return os.strerror(e.errno)

def __get_file_hash(file_path):
    try:
        with open(__to_fspath(file_path), 'rb') as f:
            hash = hashlib.md5()
            #buffer_size = 65536
            buffer_size = hash.block_size * 1024
            while True:
                chunk = f.read(buffer_size)
                if not chunk:
                    break
                hash.update(chunk)
        return hash.hexdigest()
    except IOError as e:
        if e.errno == errno.EISDIR:
            return HASH_DIRECTORY
        elif e.errno == errno.ENOENT:
            return HASH_NOT_FOUND
        return os.strerror(e.errno)

def _verify_files(dir1_path, dir2_path, relpath, log, dir1_alt_name, dir2_alt_name, check_hash):
    result = 0
    if relpath:
        dir1_abspath = os.path.join(dir1_path, relpath)
        dir2_abspath = os.path.join(dir2_path, relpath)
    else:
        dir1_abspath = dir1_path
        dir2_abspath = dir2_path

    dir1_list = os.listdir(__to_fspath(dir1_abspath))
    dir2_list = os.listdir(__to_fspath(dir2_abspath))
    merged_list = sorted(set(dir1_list + dir2_list))
    for file in merged_list:
        if relpath:
            file_relpath = os.path.join(relpath, file)
        else:
            file_relpath = file

        file1_abspath = os.path.join(dir1_abspath, file)
        file2_abspath = os.path.join(dir2_abspath, file)

        file1_stat = __get_file_stat(file1_abspath)
        file2_stat = __get_file_stat(file2_abspath)

        if file1_stat == HASH_DIRECTORY and file2_stat == HASH_DIRECTORY:
            if _verify_files(dir1_path, dir2_path, file_relpath, log, dir1_alt_name, dir2_alt_name, check_hash) != 0:
                result = 1
            continue

        if check_hash:
            file1_hash = __get_file_hash(file1_abspath)
            file2_hash = __get_file_hash(file2_abspath)

            if file1_hash != file2_hash:
                result = 1
                if log:
                    log.log(LOG_FMT_VERIFY_NG_STR % (file_relpath, FILE_VERIFICATION_NG, file1_hash, file2_hash))
                continue

            if file1_stat != file2_stat:
                result = 1
                if log:
                    log.log(LOG_FMT_VERIFY_OK_STR % (file_relpath, FILE_VERIFICATION_NG, file1_hash))
                continue

            if log:
                log.log(LOG_FMT_VERIFY_OK_STR % (file_relpath, FILE_VERIFICATION_OK, file1_hash))
        else:
            if file1_stat != file2_stat:
                result = 1
                if log:
                    log.log(LOG_FMT_STR % (file_relpath, FILE_VERIFICATION_NG))
                continue

            if log:
                log.log(LOG_FMT_STR % (file_relpath, FILE_VERIFICATION_OK))

    return result

def __verify_files(dev, dir_path, log, dev_alt_name, src_alt_name, check_hash):
    __makedirs(ROOTFS_MNT, exist_ok=True)
    returncode, outs = __exec_cmd([MOUNT_CMD, "-o", "ro,noatime,unhide", dev, ROOTFS_MNT])
    if returncode !=0:
        print("mount failed", file=sys.stderr)
        return 1
    result = _verify_files(ROOTFS_MNT, dir_path, None, log, dir1_alt_name=dev_alt_name, dir2_alt_name=src_alt_name, check_hash=check_hash)
    returncode, outs = __exec_cmd([UMOUNT_CMD, ROOTFS_MNT])
    if returncode !=0:
        print("umount failed", file=sys.stderr)
        return 1
    return result

def verify(dev, lock_file, dir_path, dev_alt_name="dev", src_alt_name="src", check_hash=False):
    if not dev:
        dev = DEV_DEFAULT
    if not lock_file:
        lock_file = LOCK_FILE_DEFAULT
    flocker = FileLocker(lock_file, True)
    dev_status = get_dev_status(dev)
    if not dev_status:
        print("get_dev_status failed", file=sys.stderr)
        return 1
    if dev_status != DEV_STATUS_READY:
        print("dev_status may not ready", file=sys.stderr)
        return 1
    volid = get_volid(dev)
    if not volid:
        print("not volid", file=sys.stderr)
        return 1
    start_time = time.time()
    from bdreclog import BdrecLog
    process_log = BdrecLog(file=sys.stdout)
    if not process_log:
        print("not process_log", file=sys.stderr)
        return 1
    result = 0
    process_log.log("volid: %s" % volid)
    process_log.log("datetime: %s" % datetime.fromtimestamp(time.time()))
    process_log.log("### Verify Start ###")
    if __verify_files(dev, dir_path, log=process_log, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, check_hash=check_hash) != 0:
        print("__verify_files failed", file=sys.stderr)
        result = 1
    end_time = time.time()
    elapsed_time = end_time - start_time
    process_log.log("datetime: %s" % datetime.fromtimestamp(end_time))
    if result == 0:
        process_log.log("%s: %s" % (volid, BdrecLog.SUCCESS))
    else:
        process_log.log("%s: %s" % (volid, BdrecLog.FAILED))
    return result

def __write(dev, lock_file, volid, dir_path, close_disc, log_dir_path, verify, check_hash, dev_alt_name, src_alt_name, func_started=None, func_finished=None):
    flocker = FileLocker(lock_file, True)
    if func_started:
        func_started()
    if check_writable_volid(volid) != 0:
        print("check_writable_volid failed", file=sys.stderr)
        return 1, REC_STATUS_FAILED
    dev_status = get_dev_status(dev)
    if not dev_status:
        print("get_dev_status failed", file=sys.stderr)
        return 1, REC_STATUS_FAILED
    if dev_status != DEV_STATUS_READY:
        print("dev_status may not ready", file=sys.stderr)
        return 1, REC_STATUS_FAILED
    if __set_rec_status(dev, REC_STATUS_RECORDING) != 0:
        print("__set_rec_status failed", file=sys.stderr)
        return 1, REC_STATUS_FAILED
    start_time = time.time()
    process_log = None
    if log_dir_path:
        log_dir_with_volid_path = os.path.join(log_dir_path, volid)
        __makedirs(log_dir_with_volid_path, exist_ok=True)
        if os.getuid() == 0:
            try:
                from ionas.backupsync.logging import ProcessLog
                process_log = ProcessLog(__to_fspath(log_dir_with_volid_path), line_sep="\r\n")
            except:
                print("WARNING: cannot use ionas logging", file=sys.stderr)
        if not process_log:
            print("WARNING: cannot to write process log", file=sys.stderr)
    if process_log:
        process_log.open(start_epoch=start_time)
        process_log.log("volid: %s" % volid)
        process_log.log("datetime: %s" % datetime.fromtimestamp(start_time))
        process_log.log("### Write Start ###")
    cmd = [GROWISOFS_CMD, "-Z", dev, "-udf", "-volid", volid, "-graft-points", dir_path ]
    # workaround for "FATAL: /dev/dvd already carries isofs!"
    cmd += ["-use-the-force-luke=tty"]
    # workaround for M-DISC that is identical normal BD-R.
    cmd += ["-speed=4"]
    if close_disc:
        cmd += ["-dvd-compat"]
    returncode, outs = __exec_cmd(cmd)
    ng_file_list = None
    result = 0
    rec_status = REC_STATUS_SUCCESS
    if returncode != 0:
        rec_status = REC_STATUS_FAILED
        print("%s failed" % (GROWISOFS_CMD), file=sys.stderr)
        ng_file_list = []
        result = 1
    # clean up to workaround for M-DISC.
    returncode, outs = __exec_cmd([EJECT_CMD, dev, "--cdspeed", "0"])
    if returncode != 0:
        rec_status = REC_STATUS_FAILED
        print("%s failed" % (EJECT_CMD), file=sys.stderr)
        ng_file_list = []
        result = 1
    if rec_status == REC_STATUS_SUCCESS:
        if verify:
            if process_log:
                process_log.log("datetime: %s" % datetime.fromtimestamp(time.time()))
                process_log.log("### Verify Start ###")
            if __verify_files(dev, dir_path, log=process_log, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, check_hash=check_hash) != 0:
                print("__verify_files failed", file=sys.stderr)
                result = 1
                print("change rec_status to failed", file=sys.stderr)
                rec_status = REC_STATUS_FAILED
        else:
            if process_log:
                __list_files(dir_path, log=process_log, dev_alt_name=dev_alt_name)
    end_time = time.time()
    elapsed_time = end_time - start_time
    if process_log:
        process_log.log("datetime: %s" % datetime.fromtimestamp(end_time))
        if result == 0:
            process_log.log("%s: %s" % (volid, ProcessLog.SUCCESS))
            process_log.close(ProcessLog.SUCCESS)
        else:
            process_log.log("%s: %s" % (volid, ProcessLog.FAILED))
            process_log.close(ProcessLog.FAILED)
    if log_dir_path:
        last_written_summary = {
                "volid": volid,
                "close_disc": close_disc,
                "start_time": str(datetime.fromtimestamp(start_time)),
                "end_time": str(datetime.fromtimestamp(end_time)),
                "elapsed_time": str(elapsed_time),
                "result": result,
                }
        if __set_last_written_summary(volid, log_dir_path, last_written_summary) != 0:
            print("__set_last_written_summary failed", file=sys.stderr)
            result = 1
    if func_finished:
        func_finished()
    if __set_rec_status(dev, rec_status) != 0:
        print("__set_rec_status failed", file=sys.stderr)
        result = 1
    return result, rec_status

def __set_last_written_summary(volid, log_dir_path, last_written_summary):
    log_dir_with_volid_path = os.path.join(log_dir_path, volid)
    __makedirs(log_dir_with_volid_path, exist_ok=True)
    if not os.path.isdir(__to_fspath(log_dir_with_volid_path)):
        return 1
    last_written_summary_path = os.path.join(log_dir_with_volid_path,
                                             LASTWRITTENSUMMARY)
    flocker = FileLocker(last_written_summary_path, lock=True)
    try:
        flocker.write(__to_keyvalue_str(last_written_summary))
    except:
        return 1
    return 0

def get_last_written_summary(volid, log_dir_path):
    log_dir_with_volid_path = os.path.join(log_dir_path, volid)
    if not os.path.isdir(__to_fspath(log_dir_with_volid_path)):
        return None
    last_written_summary_path = os.path.join(log_dir_with_volid_path,
                                             LASTWRITTENSUMMARY)
    if not os.path.exists(__to_fspath(last_written_summary_path)):
        return None
    flocker = FileLocker(last_written_summary_path, lock=True)
    try:
        last_written_summary = __to_keyvalue(flocker.read())
    except:
        return None
    return last_written_summary

START_SUCCESS = 0
START_FAILED = 1
START_AFTER_FORKED = 128
#START_AFTER_FORKED_SUCCESS = START_AFTER_FORKED | START_SUCCESS
START_AFTER_FORKED_FAILED = START_AFTER_FORKED | START_FAILED

def start(dev, lock_file, volid, dir_path, close_disc, log_dir_path, finished_handler_path, rec_type, verify=False, check_hash=False, dev_alt_name="dev", src_alt_name="src", force=False, func_started=None, func_finished=None):
    if not dev:
        dev = DEV_DEFAULT
    if not lock_file:
        lock_file = LOCK_FILE_DEFAULT
    dev_status = get_dev_status(dev)
    if not dev_status:
        print("get_dev_status failed", file=sys.stderr)
        return START_FAILED
    if dev_status != DEV_STATUS_READY:
        print("dev_status not ready", file=sys.stderr)
        return START_FAILED
    rec_status = get_rec_status(dev)
    if rec_status == REC_STATUS_RECORDING:
        if not force:
            print("rec_status may still recording", file=sys.stderr)
            print("use force option if not so", file=sys.stderr)
            return START_FAILED
    if __set_rec_status(dev, REC_STATUS_READY) != 0:
        print("__set_rec_status failed", file=sys.stderr)
        return START_FAILED
    rec_status = get_rec_status(dev)
    if rec_status != REC_STATUS_READY:
        print("rec_status not ready", file=sys.stderr)
        return START_FAILED

    if not finished_handler_path:
        print(REC_STATUS_RECORDING, file=sys.stderr)
        result, rec_status = __write(dev, lock_file, volid, dir_path, close_disc, log_dir_path, verify, check_hash=check_hash, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, func_started=None, func_finished=func_finished)
        print(str(rec_status), file=sys.stderr)
        if result != 0:
            return START_FAILED
        return START_SUCCESS

    child_pid = os.fork()
    if child_pid == 0:
        result, rec_status = __write(dev, lock_file, volid, dir_path, close_disc, log_dir_path, verify, check_hash=check_hash, dev_alt_name=dev_alt_name, src_alt_name=src_alt_name, func_started=func_started, func_finished=func_finished)
        cmd = [finished_handler_path, rec_status, rec_type]
        returncode, outs = __exec_cmd(cmd)
        if returncode != 0:
            print("%s failed" % (finished_handler_path), file=sys.stderr)
            result = START_AFTER_FORKED_FAILED
        os._exit(result)
    while rec_status == REC_STATUS_READY:
        time.sleep(1)
        rec_status = get_rec_status(dev)
        if not rec_status:
            print("get_rec_status failed", file=sys.stderr)
            return START_AFTER_FORKED_FAILED
    if rec_status != REC_STATUS_RECORDING:
        print("rec_status not recording", file=sys.stderr)
        return START_AFTER_FORKED_FAILED
    return START_SUCCESS

def get_dev_status(dev):
    if not dev:
        dev = DEV_DEFAULT
    flocker = FileLocker(FILE_DEV_STATUS, True)
    cmd = [SG_TURS_CMD, dev]
    returncode, outs = __exec_cmd(cmd)
    if returncode != 0:
        return DEV_STATUS_NOTREADY
    return DEV_STATUS_READY

def get_rec_status(dev):
    if not dev:
        dev = DEV_DEFAULT
    rec_status = None
    flocker = FileLocker(FILE_REC_STATUS, True)
    file_status = flocker.read()
    if file_status:
        status_list = file_status.split(',')
        if status_list[0] != dev:
            print("%s not match %s" % (status_list[0], dev), file=sys.stderr)
            return None
        rec_status = status_list[1]
    else:
        flocker.write(','.join((dev, REC_STATUS_READY)))
        rec_status = REC_STATUS_READY
    return rec_status

def __set_rec_status(dev, status):
    flocker = FileLocker(FILE_REC_STATUS, True)
    if not status:
        return 1
    try:
        flocker.write(','.join((dev,status)))
    except:
        return 1
    return 0

def __get_mediainfo(dev):
    cmd = [MEDIAINFO_CMD, dev]
    returncode, outs = __exec_cmd(cmd)
    if returncode != 0:
        return None
    mediainfo = {}
    for line in __to_str(outs).split("\n"):
        key_value = line.split(":")
        if len(key_value) == 2:
            key = key_value[0].strip()
            value = key_value[1].strip()
            mediainfo[key] = value
    return mediainfo

def __get_mountedmedia(mediainfo):
    if not mediainfo:
        return None
    if "Mounted Media" not in mediainfo:
        return None
    try:
        mountedmedia = mediainfo["Mounted Media"].split(",")
    except:
        return None
    return mountedmedia

def get_mountedmedia(dev):
    if not dev:
        dev = DEV_DEFAULT
    mediainfo = __get_mediainfo(dev)
    mountedmedia = __get_mountedmedia(mediainfo)
    return mountedmedia

def get_disc_status(dev):
    if not dev:
        dev = DEV_DEFAULT
    mediainfo = __get_mediainfo(dev)
    if not mediainfo:
        return None
    mountedmedia = __get_mountedmedia(mediainfo)
    if not mountedmedia:
        return None
    if mountedmedia[0] not in SUPPORTEDMEDIA:
        return DISC_STATUS_UNSUPPORTEDMEDIA
    if "Disc status" not in mediainfo:
        return None
    disc_status = mediainfo["Disc status"]
    return disc_status

def __get_udfinfo(dev):
    cmd = [UDFINFO_CMD, dev]
    returncode, outs = __exec_cmd(cmd)
    if returncode != 0:
        return None
    udfinfo = {}
    for line in __to_str(outs).split("\n"):
        key_value = line.split("=")
        if len(key_value) == 2:
            key = key_value[0].strip()
            value = key_value[1].strip()
            udfinfo[key] = value
    return udfinfo

def get_volid(dev):
    if not dev:
        dev = DEV_DEFAULT
    udfinfo = __get_udfinfo(dev)
    if not udfinfo:
        return None
    if "label" not in udfinfo:
        return None
    return udfinfo["label"]

def __eject(dev, close_tray=False):
    cmd = [EJECT_CMD, dev]
    if close_tray:
        cmd += ["-T"]
    returncode, outs = __exec_cmd(cmd)
    if returncode != 0:
        return 1
    return 0

def eject(dev, close_tray=False, force=False):
    if not dev:
        dev = DEV_DEFAULT
    rec_status = get_rec_status(dev)
    if rec_status == REC_STATUS_RECORDING:
        if not force:
            print("rec_status may still recording", file=sys.stderr)
            print("use force option if not so", file=sys.stderr)
            return 1
        if __set_rec_status(dev, REC_STATUS_FAILED) != 0:
            print("__set_rec_status failed", file=sys.stderr)
            return 1
        rec_status = get_rec_status(dev)
        if rec_status != REC_STATUS_FAILED:
            print("rec_status not ready", file=sys.stderr)
            return 1
    if __eject(dev, close_tray) != 0:
        return 1
    return 0

if __name__ == '__main__':
    __init()
    usage = "usage: %prog [options] get_rec_status|get_dev_status|start|get_mountedmedia|get_disc_status|get_volid|eject|close_tray|get_last_written_summary|check_writable_volid"
    parser = OptionParser(usage)
    parser.add_option("-d", "--dev", dest="dev", help="dev", default=None, action="store", type="string")
    parser.add_option("-l", "--lock_file", dest="lock_file", help="lock_file", default=None, action="store", type="string")
    parser.add_option("-i", "--volid", dest="volid", help="volid", default=None, action="store", type="string")
    parser.add_option("-p", "--dir_path", dest="dir_path", help="dir_path", default=None, action="store", type="string")
    parser.add_option("-c", "--close_disc", dest="close_disc", help="close_disc", default=False, action="store_true")
    parser.add_option("-L", "--log_dir_path", dest="log_dir_path", help="log_dir_path", default=None, action="store", type="string")
    parser.add_option("-D", "--dev_alt_name", dest="dev_alt_name", help="dev_alt_name", default=None, action="store", type="string")
    parser.add_option("-S", "--src_alt_name", dest="src_alt_name", help="src_alt_name", default=None, action="store", type="string")
    parser.add_option("-H", "--finished_handler_path", dest="finished_handler_path", help="finished_handler_path", default=None, action="store", type="string")
    parser.add_option("-t", "--rec_type", dest="rec_type", help="rec_type", default=None, action="store", type="string")
    parser.add_option("-v", "--verify", dest="verify", help="verify", default=False, action="store_true")
    parser.add_option("-C", "--check_hash", dest="check_hash", help="check_hash", default=False, action="store_true")
    parser.add_option("-f", "--force", dest="force", help="force", default=False, action="store_true")
    (options, args) = parser.parse_args()

    if not args or len(args) != 1:
        parser.print_help()
        exit(1)

    command = args[0]

    if command == "get_rec_status":
        rec_status = get_rec_status(options.dev)
        if not rec_status:
            exit(1)
        print(rec_status)
        exit(0)
    elif command == "get_dev_status":
        rec_status = get_dev_status(options.dev)
        if not rec_status:
            exit(1)
        print(rec_status)
        exit(0)
    elif command == "start":
        if not options.volid:
            print("Require --volid.")
            exit(1)
        if not options.dir_path:
            print("Require --dir_path.")
            exit(1)
        if start(options.dev, options.lock_file, options.volid, options.dir_path, options.close_disc, options.log_dir_path, options.finished_handler_path, options.rec_type, verify=options.verify, check_hash=options.check_hash, dev_alt_name=options.dev_alt_name, src_alt_name=options.src_alt_name, force=options.force) != 0:
            exit(1)
        exit(0)
    elif command == "get_mountedmedia":
        mountedmedia = get_mountedmedia(options.dev)
        if not mountedmedia:
            exit(1)
        print(",".join(mountedmedia))
        exit(0)
    elif command == "get_disc_status":
        disc_status = get_disc_status(options.dev)
        if not disc_status:
            exit(1)
        print(disc_status)
        exit(0)
    elif command == "get_volid":
        volid = get_volid(options.dev)
        print(volid)
        exit(0)
    elif command == "eject":
        if eject(options.dev, force=options.force) != 0:
            exit(1)
        exit(0)
    elif command == "close_tray":
        if eject(options.dev, close_tray=True, force=options.force) != 0:
            exit(1)
        exit(0)
    elif command == "get_last_written_summary":
        if not options.volid:
            print("Require --volid.")
            exit(1)
        if not options.log_dir_path:
            print("Require --log_dir_path.")
            exit(1)
        last_written_summary = get_last_written_summary(volid=options.volid, log_dir_path=options.log_dir_path)
        if not last_written_summary:
            exit(1)
        print(__to_keyvalue_str(last_written_summary), end="")
        exit(0)
    elif command == "check_writable_volid":
        if not options.volid:
            print("Require --volid.")
            exit(1)
        if check_writable_volid(options.volid) != 0:
            exit(1)
        exit(0)
    elif command == "list":
        if list(dev=options.dev, dev_alt_name=options.dev_alt_name) != 0:
            exit(1)
        exit(0)
    elif command == "verify":
        if not options.dir_path:
            print("Require --dir_path.")
            exit(1)
        if verify(dev=options.dev, lock_file=options.lock_file, dir_path=options.dir_path, dev_alt_name=options.dev_alt_name, src_alt_name=options.src_alt_name) != 0:
            exit(1)
        exit(0)

    exit(1)
