# -*- coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import absolute_import
from __future__ import division

import os
import sys
import datetime
from subprocess import Popen, PIPE
from product_conf import SUDO_CMD as SUDO

from .cached_property import cached_property
from .gettext import gettext
from project.fal.led import LED
from project.fal.buzzer import Buzzer

# Add "/usr/local/bin/meimeikun" to sys.path to import statemgr.
sys.path.append("/usr/local/bin/meimeikun")
from statemgr import set_busy_status, unset_busy_status


# FIXME
BDREC = '/usr/local/bin/meimeikun/bdrec/meimeikun_bdrec.py'
ON_ARCHIVED = '/usr/local/bin/meimeikun/onarchived'
ON_FINALIZED = '/usr/local/bin/meimeikun/onfinalized'
DEV_BD = '/dev/bd'

ENCODING = 'UTF-8'


class _RecAction:
    WRITE = 'write'
    CLOSE = 'write with close'


class _RecStatus:
    ready = 'ready'
    recording = 'recording'
    success = 'success'


class _DevStatus:
    READY = 'ready'


class _DiscStatus:
    BLANK = 'blank'
    UNSUPPORTED = 'unsupported media'


class Disk(object):

    @classmethod
    def is_valid_volumelabel(cls, volumelabel):
        return cls._exec_validate_volumelabel(volumelabel)

    @classmethod
    def _exec_validate_volumelabel(cls, volumelabel):
        args = [
            SUDO,
            BDREC, 'check_writable_volid',
            '-i', volumelabel,
        ]
        args = [a.encode(ENCODING) for a in args]
        proc = Popen(args)
        proc.communicate()
        ok = proc.returncode == 0
        return ok

    def __init__(self):
        pass

    @property
    def status(self):
        if self.archiving:
            return u"ディスクに書き込み中"
        if self.closing:
            return u"ディスクのクローズ処理中"
        if self.finalized:
            return u"クローズ済みディスク({0}年度)".format(self.year)
        if not self.ready:
            return u"ディスクなし"
        if self.unsupported:
            return u"非サポートディスク"
        if not self.initialized:
            return u"ディスク未フォーマット"
        return u"ディスク接続済み({0}年度)".format(self.year)

    @property
    def ready(self):
        return self._device_status == _DevStatus.READY

    @property
    def blank(self):
        return self._disc_status == _DiscStatus.BLANK

    @property
    def unsupported(self):
        return self._disc_status == _DiscStatus.UNSUPPORTED \
               or self._disc_status is None

    @property
    def initialized(self):
        return bool(self.year)

    @property
    def finalized(self):
        close_disc = self._raw_diskinfo.get('close_disc', None)
        return close_disc == 'closed'

    @property
    def archiving(self):
        status, action = self._recording_status
        return (status == _RecStatus.recording) and (action == _RecAction.WRITE)

    @property
    def closing(self):
        status, action = self._recording_status
        return (status == _RecStatus.recording) and (action == _RecAction.CLOSE)

    @cached_property
    def volumelabel(self):
        volumelabel = self._raw_diskinfo.get('volid', None)
        return volumelabel

    @cached_property
    def year(self):
        year = self._raw_diskinfo.get('fiscal_year', None)
        try:
            year = int(year)
        except Exception as e:  # ValueError
            year = None
        return year

    @cached_property
    def datetime(self):
        s = self._raw_diskinfo.get('datetime', None)
        if s:
            try:
                return datetime.datetime.strptime(s, '%Y-%m-%d %H:%M:%S.%f')
            except Exception as e:
                pass
        return None

    @property
    def initializable(self):
        if not self.ready:
            return False
        if not self.blank:
            return False
        if self.initialized:
            return False
        if self.archiving:
            return False
        if self.closing:
            return False
        return True

    @property
    def archivable(self):
        if not self.ready:
            return False
        if not self.initialized:
            return False
        if self.archiving:
            return False
        if self.closing:
            return False
        if self.finalized:
            return False
        return True

    @property
    def finalizable(self):
        return self.archivable

    @property
    def ejectable(self):
        return self.ready or self.unsupported

    def initialize(self, **kwargs):
        year = kwargs['year']
        volumelabel = kwargs['volumelabel']
        return self._exec_initialize(year, volumelabel)

    def archive(self):
        if not set_busy_status():
            msg = gettext('ERR_MEIMEIKUN_STATE_CHANGE')
            Buzzer().error()
            return False, msg

        try:
            LED().status(True, blink=True, color="green")
            started, msg = self._exec_archive_start()
            LED().status(True, blink=False, color="green")
            if started:
                Buzzer().ok()
            else:
                Buzzer().error()
                unset_busy_status()
            return started, msg
        except Exception as err:
            unset_busy_status()
            raise

    def finalize(self):
        if not set_busy_status():
            msg = gettext('ERR_MEIMEIKUN_STATE_CHANGE')
            Buzzer().error()
            return False, msg
        try:
            LED().status(True, blink=True, color="green")
            started, msg = self._exec_finalize_start()
            LED().status(True, blink=False, color="green")
            if started:
                Buzzer().ok()
            else:
                Buzzer().error()
                unset_busy_status()
            return started, msg
        except Exception as err:
            unset_busy_status()
            raise

    def eject(self):
        return self._exec_eject()

    @cached_property
    def _raw_diskinfo(self):
        return self._exec_obtain_diskinfo()

    @cached_property
    def _recording_status(self):
        return self._exec_obtain_recording_status()

    @cached_property
    def _device_status(self):
        return self._exec_obtain_device_status()

    @cached_property
    def _disc_status(self):
        return self._exec_obtain_disc_status()

    def _exec_obtain_recording_status(self):
        """
        e.g.
            return "success", "write"
        """
        args = [SUDO, BDREC, 'rec_status']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        if proc.returncode != 0:
            return None, None
        status = stdout.decode(ENCODING).strip()
        if status == _RecStatus.ready:
            return status, None
        status, action = [s.strip() for s in status.split(',')]
        return status, action

    def _exec_obtain_device_status(self):
        args = [SUDO, BDREC, 'dev_status']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        if proc.returncode != 0:
            return None
        status = stdout.decode(ENCODING).strip()
        return status

    def _exec_obtain_disc_status(self):
        args = [SUDO, BDREC, 'disc_status']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        if proc.returncode != 0:
            return None
        status = stdout.decode(ENCODING).strip()
        return status

    def _exec_obtain_diskinfo(self):
        args = [SUDO, BDREC, 'get_format']
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        _raw_diskinfo = {}
        for kv in stdout.decode(ENCODING).splitlines():
            try:
                k, v = kv.split('=', 1)
                _raw_diskinfo[k] = v
            except Exception as err:
                continue
        return _raw_diskinfo

    def _exec_initialize(self, year, volumelabel):
        args = [
            SUDO,
            BDREC,
            'format',
            # '--dev', DEV_BD,
            '--volid', volumelabel,
            '--fiscal_year', unicode(year),
        ]
        args = [a.encode(ENCODING) for a in args]
        proc = Popen(args, stdout=PIPE, stderr=PIPE)
        stdout, stderr = proc.communicate()
        ok = proc.returncode == 0

        cn = {
            0: 'MSG_MEIMEIKUN_DISK_INITIALIZED',
            8: 'ERR_MEIMEIKUN_MADE_BY_SOMEONE_DISK',
        }.get(proc.returncode, 'ERR_MEIMEIKUN_INITIALIZE')
        msg = gettext(cn)
        return ok, msg

    def _exec_archive_start(self, B_STARTED=0b10000000):
        dirname = _obtain_year_dirname(self.year)
        args = [
            SUDO,
            BDREC, 'write',
            '--fiscal_year', unicode(self.year),
            '--dir_path', dirname,
            '--finished_handler_path', ON_ARCHIVED,
        ]
        args = [a.encode(ENCODING) for a in args]
        proc = Popen(args)
        proc.communicate()
        ok = proc.returncode == 0
        started = ok or bool(proc.returncode & B_STARTED)
        statuscode = 0 if started else proc.returncode & ~B_STARTED
        cn = {
            0: 'MSG_MEIMEIKUN_START_MANUALLY_DISK_WRITE',
            8: 'ERR_MEIMEIKUN_MADE_BY_SOMEONE_DISK',
        }.get(statuscode, 'ERR_MEIMEIKUN_DISK_WRITE')
        msg = gettext(cn)
        return started, msg

    def _exec_finalize_start(self, B_STARTED=0b10000000):
        dirname = _obtain_year_dirname(self.year)
        args = [
            SUDO,
            BDREC, 'write',
            '--fiscal_year', unicode(self.year),
            '--dir_path', dirname,
            '--close_disc',
            '--finished_handler_path', ON_FINALIZED,
        ]
        args = [a.encode(ENCODING) for a in args]
        proc = Popen(args)
        proc.communicate()
        ok = proc.returncode == 0
        started = ok or bool(proc.returncode & B_STARTED)
        statuscode = 0 if started else proc.returncode & ~B_STARTED
        cn = {
            0: 'MSG_MEIMEIKUN_START_DISK_FINALIZE',
            8: 'ERR_MEIMEIKUN_MADE_BY_SOMEONE_DISK',
        }.get(statuscode, 'ERR_MEIMEIKUN_DISK_FINALIZE')
        msg = gettext(cn)
        return started, msg

    def _exec_eject(self):
        args = [
            SUDO,
            BDREC, 'eject',
        ]
        args = [a.encode(ENCODING) for a in args]
        proc = Popen(args)
        proc.communicate()
        ok = proc.returncode == 0
        return ok


def _obtain_year_dirname(year):
    from .._utils import get_path_of_protected_fiscal_year_folder
    return get_path_of_protected_fiscal_year_folder(year).decode(ENCODING).rstrip()
