# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from project.view import FuncView, MenuView
from root.modules.form import BaseForm
from pkgaddon.meimeikun_config import KEEP_SHARE_NAME

import os
import datetime
from ionas.request import Entry
from . import fields
from . import _utils
from fal_info import (FAL_SHARES_PATH,)
from project.forms.fields.extras.share import ShareNameValidator
from django.core.exceptions import ValidationError

from .disk.views import MeimeikunDiskFormatForm, Disk


def _encode_str(text):
    if isinstance(text, unicode):
        return text.encode('utf-8')
    else:
        return text


def _is_initialized():
    config = _utils.load_config()
    if config.get("fiscal_year", None) is None \
            or config.get("closing_date", None) is None \
            or config.get("timestamp_share", None) is None \
            or config.get("prekeep_share", None) is None \
            or config.get("schedule_archive", None) is None:
        return False

    return True


def _get_exist_shares():
    shares = []
    try:
        entry = Entry('load', '<shares />')
        entry = _utils.send_entry(FAL_SHARES_PATH, entry)

        for share_node in entry.get().get_node_list('sharelist/share'):
            if share_node.get_attr_value('undeletable', False):
                continue
            share_name = share_node.get_node('name').get_text()
            shares.append(share_name)
    except Exception as err:
        pass
    return shares


class MeimeikunMenuView(MenuView):
    def view(self, request, *args, **kwargs):
        if not _is_initialized():
            return {self.KEY_REDIRECT: './init'}  # FIXME: reverse()
        return super(MeimeikunMenuView, self).view(request, *args, **kwargs)


class MeimeikunMenuInitForm(BaseForm):
    def __init__(self, **kwargs):
        super(MeimeikunMenuInitForm, self).__init__(**kwargs)

    def clean(self):
        cleaned_data = self.cleaned_data
        closing_month = cleaned_data.get('closing_month')
        closing_day = cleaned_data.get('closing_day')
        timestamp_share = _encode_str(cleaned_data.get('timestamp_share'))
        prekeep_share = _encode_str(cleaned_data.get('prekeep_share'))

        schedule_enabled = cleaned_data.get('enabled', False)
        dom = cleaned_data.get('dom', None)
        tod = cleaned_data.get('tod', None)

        if not self.validate_date(int(closing_month), int(closing_day)):
            self._errors['closing_day'] = self.error_class(["存在しない日付が指定されています。"])

        if prekeep_share is not None and timestamp_share == prekeep_share:
            self._errors['timestamp_share'] = self.error_class(["同じフォルダーは設定できません。"])
            self._errors['prekeep_share'] = self.error_class(["同じフォルダーは設定できません。"])

        exist_shares = _get_exist_shares()

        if timestamp_share not in exist_shares:
            try:
                self.validate_sharename(timestamp_share)
            except Exception as err:
                self._errors['timestamp_share'] = self.error_class(err.messages)

        if prekeep_share not in exist_shares:
            try:
                self.validate_sharename(prekeep_share)
            except Exception as err:
                self._errors['prekeep_share'] = self.error_class(err.messages)

        if schedule_enabled:
            if dom is None:
                self._errors['dom'] = self.error_class([_(u"This field is required.")])
            if tod is None:
                self._errors['tod'] = self.error_class([_(u"This field is required.")])

        return cleaned_data

    def validate_sharename(self, sharename):
        try:
            validator = ShareNameValidator()
            validator(sharename)
        except ValidationError as err:
            raise
        except Exception as err:
            raise ValidationError(_(u'UnknownError'))

    def validate_date(self, month, day):
        try:
            year = datetime.datetime.today().year
            datetime.datetime(year, month, day)
        except Exception as err:
            return False

        if month == 2 and day == 29:
            return False

        return True

    fiscal_year = fields.MeimeikunFiscalYearField()
    closing_month = fields.MeimeikunClosingMonthField()
    closing_day = fields.MeimeikunClosingDayField()
    timestamp_share = fields.MeimeikunShareNameSetField(
        label="署名フォルダー", initial="命名くん署名用")
    prekeep_share = fields.MeimeikunShareNameSetField(
        label="保存先フォルダー", initial="命名くん保存先")

    enabled = fields.MeimeikunArchiveScheduleEnabledField()
    dom = fields.MeimeikunArchiveScheduleDayOfMonthField(required=False)
    tod = fields.MeimeikunArchiveScheduleTimeOfDayField(initial=datetime.time(23, 55), required=False)


class MeimeikunMenuInitView(FuncView):

    views = ['apply', 'format']

    def view(self, request):
        init_form = MeimeikunMenuInitForm()
        format_form = MeimeikunDiskFormatForm()
        disk = Disk()

        template = os.path.join(self.TEMPLATES, 'init.html')
        dictionary = {
            'init_form': init_form,
            'format_form': format_form,
            'shares': _get_exist_shares(),
            'disk': disk,
            'config': {"keep_share" : KEEP_SHARE_NAME},
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_apply(self, request):
        init_form = MeimeikunMenuInitForm(data=request.POST)
        format_form = MeimeikunDiskFormatForm()
        form = init_form
        disk = Disk()

        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                cleaned_data = form.cleaned_data
                config_dict = self.make_config_dict(cleaned_data)
                succeeded, message = _utils.update_config(config_dict)
                proc.result(succeeded)
                self.set_message(message)

        if _is_initialized():
            return {
                self.KEY_REDIRECT: './menu',  # FIXME: reverse()
            }

        template = os.path.join(self.TEMPLATES, 'init.html')
        dictionary = {
            'init_form': form,
            'format_form': format_form,
            'shares': _get_exist_shares(),
            'disk': disk,
            'config': {"keep_share" : KEEP_SHARE_NAME},
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def view_format(self, request):
        init_form = MeimeikunMenuInitForm()
        format_form = MeimeikunDiskFormatForm(data=request.POST)
        form = format_form
        disk = Disk()

        with self.processing(constname='ERR_MEIMEIKUN_STATE_CHANGE') as proc:
            proc.break_if_state_change()
            if form.is_valid():
                year = form.cleaned_data['year']
                volumelabel = form.cleaned_data['volumelabel']
                succeeded, message = disk.initialize(
                    year=year,
                    volumelabel=volumelabel,
                )
                proc.result(succeeded)
                self.set_message(message)
                disk = Disk()

        template = os.path.join(self.TEMPLATES, 'init.html')
        dictionary = {
            'form': form,
            'init_form': init_form,
            'format_form': format_form,
            'shares': _get_exist_shares(),
            'disk': disk,
        }
        return {
            self.KEY_TEMPLATE: template,
            self.KEY_TEMPLATE_DIC: dictionary,
        }

    def make_config_dict(self, cleaned_data):
        day = cleaned_data.get("dom", None)
        day = int(day) if day else None
        tod = cleaned_data.get("tod", None)
        hour = tod.hour if isinstance(tod, datetime.time) else None
        minute = tod.minute if isinstance(tod, datetime.time) else None

        config_dict = {
            "fiscal_year": cleaned_data["fiscal_year"],
            "closing_date": {
                "month": int(cleaned_data["closing_month"]),
                "day": int(cleaned_data["closing_day"])
            },
            "timestamp_share": cleaned_data["timestamp_share"],
            "prekeep_share": cleaned_data["prekeep_share"],
            "schedule_archive": {
                "enabled": cleaned_data["enabled"],
                "day": day,
                "time": {
                    "hour": hour,
                    "minute": minute
                }
            }
        }
        return config_dict
